-- FUNCTION: SafetyChecks.vhd
-- AUTHOR:   Christos Zamantzas
-- CREATED:  25 June 2009
-- VERSION:  3

-- Changes in ver.2: Initial conversion to vhdl.
-- Changes in ver.3: No semantic changes. Rename of signals.

LIBRARY ieee;
USE ieee.std_logic_1164.all; 

LIBRARY work;

ENTITY SafetyChecks IS 
   PORT
   (
    BLECF_A       : IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
    BLECF_B       : IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
    CID_AB        : IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
    CID_CD        : IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
    
    BLETC_SN_LSB  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    BLETC_SN_MSB  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    SN_Data       : IN  STD_LOGIC_VECTOR(63 DOWNTO 0);
    
    BLETC_FV_DB   : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    BLETC_FV      : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    
    clk           : IN STD_LOGIC;

    CID_A_OK      : OUT STD_LOGIC;
    CID_B_OK      : OUT STD_LOGIC;
    SN_OK         : OUT STD_LOGIC;
    FV_OK         : OUT STD_LOGIC
   );
END SafetyChecks;

ARCHITECTURE RTL OF SafetyChecks IS 

   COMPONENT compare_cid
      PORT(
          clock : IN STD_LOGIC;
          dataa : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
          datab : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
          aneb  : OUT STD_LOGIC
      );
   END COMPONENT;

   COMPONENT compare_serial
      PORT(
          clock : IN STD_LOGIC;
          dataa : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
          datab : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
          aneb  : OUT STD_LOGIC
      );
   END COMPONENT;

   COMPONENT compare_fv
      PORT(
          clock : IN STD_LOGIC;
          dataa : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
          datab : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
          aneb  : OUT STD_LOGIC
      );
   END COMPONENT;

SIGNAL BLETC_SN_MSB_OK :  STD_LOGIC;
SIGNAL BLETC_SN_LSB_OK :  STD_LOGIC;

BEGIN 

   inst76 : compare_cid
   PORT MAP(
          clock => clk,
          dataa => CID_AB,
          datab => BLECF_A,
          aneb => CID_a_OK);

   inst77 : compare_cid
   PORT MAP(
          clock => clk,
          dataa => CID_CD,
          datab => BLECF_B,
          aneb => CID_b_OK);

   inst84 : compare_serial
   PORT MAP(
          clock => clk,
          dataa => BLETC_SN_MSB,
          datab => SN_Data(63 DOWNTO 32),
          aneb => BLETC_SN_MSB_OK);

   inst85 : compare_serial
   PORT MAP(
          clock => clk,
          dataa => BLETC_SN_LSB,
          datab => SN_Data(31 DOWNTO 0),
          aneb => BLETC_SN_MSB_OK);

   inst87 : compare_fv
   PORT MAP(
          clock => clk,
          dataa => BLETC_FV_DB,
          datab => BLETC_FV,
          aneb => FV_OK);

   SN_OK <= BLETC_SN_MSB_OK OR BLETC_SN_LSB_OK;

END RTL;