-- BEAM PERMIT LOGGING CONTROL
  
-- VERSION: 1
-- DATE: 2008/10/06
-- AUTHOR: Christos Zamantzas

-- stats:   1 x M4K  for the FIFO (could be replaced if depth = 32 (now =256), 
--          1 x M512 for the memory, 
--        110 x LE   here!. 

LIBRARY ieee;
USE ieee.std_logic_1164.all; 
USE ieee.numeric_std.all;

LIBRARY work;

ENTITY BP_LOG_CTRL IS 
	port
	(
		MEM_data_out :  IN  STD_LOGIC_VECTOR(7 downto 0);
		FIFO_data 	 :  IN  STD_LOGIC_VECTOR(7 downto 0);
		FIFO_addr 	 :  IN  STD_LOGIC_VECTOR(3 downto 0);
		FIFO_avail 	 :  IN  STD_LOGIC;
		VME_rden 	 :  IN  STD_LOGIC;
		VME_addr 	 :  IN  STD_LOGIC_VECTOR(3 downto 0);
		clk 		 :  IN  STD_LOGIC;
		MEM_data_in  :  OUT  STD_LOGIC_VECTOR(7 downto 0);
		MEM_wr_addr  :  OUT  STD_LOGIC_VECTOR(3 downto 0);
		MEM_wren 	 :  OUT  STD_LOGIC;
		MEM_rd_addr  :  OUT  STD_LOGIC_VECTOR(3 downto 0);
		MEM_rden 	 :  OUT  STD_LOGIC;
		FIFO_rdreq 	 :  OUT  STD_LOGIC		
	);
	
END BP_LOG_CTRL;

ARCHITECTURE RTL OF BP_LOG_CTRL IS 

--signals
signal read_locked 	  :std_logic;
signal read_ended 	  :std_logic;
signal temp_data_MEM  :std_logic_vector(7 downto 0);
signal temp_data_FIFO :std_logic_vector(7 downto 0);
signal temp_addr      :std_logic_vector(3 downto 0);
signal new_data		  :std_logic;

BEGIN 

main_process: process (clk)
 
--variables
variable WordCnt  :integer range 0 to 16;

 BEGIN
 if clk'event and clk='1' then
	MEM_wren 	<= '0'; -- dont write if not needed;
	FIFO_rdreq  <= '0'; -- dont read if not needed;
	MEM_rden    <= '0'; -- dont read if not needed;




   -- [housekeeping] clean (set to zero) data in memory;
	if read_ended = '1' then
	   -- cleaning memory contents;
		if WordCnt < 16 then
			MEM_data_in <= (others => '0');
			MEM_wr_addr <= std_logic_vector(to_signed((WordCnt),4));
			MEM_wren 	<= '1';
			WordCnt 	:= WordCnt + 1; -- increment counter; 
	   -- end of cleaning
		else 
			read_ended  <= '0'; -- clear to allow continuation;	
			temp_data_MEM <= (others => '0'); -- clear old MEM data
		end if;
		 
   -- writing new data;
    elsif new_data = '1' then 
        MEM_data_in <= temp_data_FIFO or temp_data_MEM;
		MEM_wr_addr <= temp_addr;
		MEM_wren 	<= '1';
		new_data 	<= '0';
			
   -- VME is reading data 
 	elsif VME_rden = '1' or read_locked = '1' then
		read_locked <= '1'; -- lock readout 
		-- readout request of the data;
		MEM_rd_addr <= VME_addr;
		MEM_rden 	<= VME_rden;
		-- finished reading of data;
		if VME_addr = "1111" then
			read_ended  <= '1';  -- flag end
			read_locked <= '0';  -- unlock readout
			WordCnt     := 0;    -- clear housekeeping counter 
		end if;
		 
   -- reading new data;
    elsif FIFO_avail = '1' and read_ended  = '0' then
		-- read data;
		temp_data_MEM	<= MEM_data_out; -- store data to use for writting data back;
		MEM_rd_addr 	<= FIFO_addr;
		MEM_rden    	<= '1';
		temp_data_FIFO 	<= FIFO_data;    -- store data to use for writting data back;
		FIFO_rdreq  	<= '1';
		
		temp_addr   	<= FIFO_addr;    -- store address to use for writting data back;
		new_data 		<= '1';
	 
	end if;
		
 end if;
	
end process main_process;

END; 