-- SRAM control
-- version 9
-- Date: 16/04/2007

-- this version is trimmed down to perform only major functions.

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.Numeric_std.all;

ENTITY SRAM_Control IS
	PORT
	(
		enable			 			: IN	STD_LOGIC;
		fifo_empty, fifo_full		: IN	STD_LOGIC;
		reset, clk   				: IN	STD_LOGIC;
		SRAM_wrtN, inc_Counter    	: OUT	STD_LOGIC;
		Fifo_RdAck1x, Fifo_RdAck2x  : OUT	STD_LOGIC;
		Sel_Data					: OUT	STD_LOGIC_VECTOR (2 DOWNTO 0)
	);
	
END SRAM_Control;

ARCHITECTURE a OF SRAM_Control IS
	TYPE STATE_TYPE IS (IDLE, 
						CHECK_FIFO, 
						READ_A1, 
						READ_A2, 
						READ_A3, 
						READ_A4, 
						READ_A5, 
						READ_A6, 
						READ_B1, 
						READ_B2, 
						READ_B3, 
						READ_B4, 
						READ_B5, 
						READ_B6);

	SIGNAL state		: STATE_TYPE;
	SIGNAL WriteN_int	: STD_LOGIC;
	
BEGIN
	PROCESS (clk, reset)
	BEGIN
		IF reset = '1' THEN
			state <= IDLE;
		ELSIF clk'EVENT AND clk = '1' THEN
			CASE state IS
				WHEN IDLE =>
					IF enable = '1' THEN
						state <= CHECK_FIFO;
					ELSE 
						inc_Counter <= '0';
						state <= IDLE;
					END IF;
				WHEN CHECK_FIFO =>
					Fifo_RdAck2x <= '0'; -- Read B next pulse end 
					IF enable = '0' THEN
						state <= IDLE;
						SRAM_wrtN <= '1';
					ELSIF fifo_full = '1' THEN
						inc_Counter <= '1'; 
						Sel_Data <= "000";
						SRAM_wrtN <= '0';
						state <= READ_A1;
					ELSE 
						state <= CHECK_FIFO; --WAS: state <= IDLE;
						SRAM_wrtN <= '1';
					END IF;

				WHEN READ_A1 =>
					Sel_Data <= "001";
					state <= READ_A2;
				WHEN READ_A2 =>
					Sel_Data <= "010";
					Fifo_RdAck1x <= '1'; -- Read A next pulse start
					state <= READ_A3;
				WHEN READ_A3 =>
					Sel_Data <= "000";
					Fifo_RdAck1x <= '0'; -- Read A next pulse end
					state <= READ_A4;
				WHEN READ_A4 =>
					Sel_Data <= "001";
					state <= READ_A5;
				WHEN READ_A5 =>
					Sel_Data <= "010";
					state <= READ_A6;
				WHEN READ_A6 =>
					Sel_Data <= "011";
					Fifo_RdAck1x <= '1'; -- Read A next pulse start
					state <= READ_B1;
					
				WHEN READ_B1 =>
					Sel_Data <= "100";
					Fifo_RdAck1x <= '0'; -- Read A next pulse end
					state <= READ_B2;	
				WHEN READ_B2 =>
					Sel_Data <= "101";
					Fifo_RdAck2x <= '1'; -- Read B next pulse start
					state <= READ_B3;	
				WHEN READ_B3 =>
					Sel_Data <= "011";
					Fifo_RdAck2x <= '0'; -- Read B next pulse end
					state <= READ_B4;
				WHEN READ_B4 =>
					Sel_Data <= "100";
					state <= READ_B5;	
				WHEN READ_B5 =>
					Sel_Data <= "101";
					state <= READ_B6;	
				WHEN READ_B6 =>
					Sel_Data <= "000";
					Fifo_RdAck2x <= '1'; -- Read B next pulse start
					inc_Counter <= '0';
				    state <= CHECK_FIFO; --NEW:16/04/2007
					
				WHEN OTHERS =>
					state <= IDLE;

			END CASE;
		END IF;
	END PROCESS;
END a;
