LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.all;
USE IEEE.STD_LOGIC_ARITH.all;
USE IEEE.STD_LOGIC_UNSIGNED.all;

-- BD_CSRequest circuit
   --
   --

ENTITY BD_CSRequest IS
   
   PORT(
       GA               : IN STD_LOGIC_VECTOR(4 DOWNTO 0);  -- Geographical Address of the card
       BD_line          : IN STD_LOGIC_VECTOR(3 DOWNTO 0);  -- Beam Dump line
       BDLine_U_en,                                         --    -//-   UNmaskable 
       BDLine_M_en,                                         --    -//-   Maskable
       SystemUnderTest,
       clk              : IN STD_LOGIC;
       Dump_UNMASKed, 
       Dump_MASKed      : OUT STD_LOGIC
       );

END BD_CSRequest;

ARCHITECTURE RTL OF BD_CSRequest IS
   
   SIGNAL iGA              : STD_LOGIC_VECTOR(4 DOWNTO 0);
   SIGNAL Parity_error     : STD_LOGIC;
   SIGNAL BD_line_int      : STD_LOGIC_VECTOR(3 DOWNTO 0);
   SIGNAL BDLine_U_en_int  : STD_LOGIC;
   SIGNAL BDLine_M_en_int  : STD_LOGIC;
   SIGNAL SUT_int          : STD_LOGIC;

BEGIN
 PROCESS (clk)
 BEGIN
    IF  rising_edge(clk) THEN
      BD_line_int       <= BD_line;
      Parity_error      <= '0';
      BDLine_U_en_int   <= BDLine_U_en;
      BDLine_M_en_int   <= BDLine_M_en;
      SUT_int           <= SystemUnderTest;
      
      IF    GA < 12 THEN iGA <= GA - 4;
      ELSIF GA < 21 THEN iGA <= GA - 5;
      ELSE  Parity_error <= '1';
      END IF;
      
      Dump_UNMASKed  <= '0';
      Dump_MASKed    <= '0';
      
      IF BD_line_int = iGA AND Parity_error = '0' AND SUT_int = '1' THEN
         IF BDLine_U_en_int = '1' THEN Dump_UNMASKed <= '1'; END IF;
         IF BDLine_M_en_int = '1' THEN Dump_MASKed <= '1'; END IF;
      END IF;
   END IF;
    
 END PROCESS;

END RTL;
