-- ===================================================================================
-- File Name: decoder_en.vhd
-- Author: Christos Zamantzas
-- Version: 2
-- ===================================================================================
-- Description: Creates an address selection signal to drive the muxes 
--                in the correct input data and a read output enable signal 
--                for when valid data are available. 
-- ===================================================================================
-- 
-- changes in ver.2: changed the non-valid output case (i.e. 'when others') 
--                      to not fall into the default address (i.e. "0000").
--
-- ===================================================================================

LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY decoder_en IS
   PORT
   (
      Enable1A   : IN STD_LOGIC;
      Enable2A   : IN STD_LOGIC;
      Enable3A   : IN STD_LOGIC;
      Enable4A   : IN STD_LOGIC;
      Enable5A   : IN STD_LOGIC;
      Enable6A   : IN STD_LOGIC;

      Enable1B   : IN STD_LOGIC;      
      Enable2B   : IN STD_LOGIC;      
      Enable3B   : IN STD_LOGIC;      
      Enable4B   : IN STD_LOGIC;
      Enable5B   : IN STD_LOGIC;
      Enable6B   : IN STD_LOGIC;      

      clk      : IN STD_LOGIC;
      Sel      : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
      en       : OUT STD_LOGIC
   );
END decoder_en;


ARCHITECTURE RTL OF decoder_en IS

SIGNAL Enable_int   : STD_LOGIC_VECTOR (11 DOWNTO 0);
BEGIN

PROCESS (clk)

BEGIN
   IF rising_edge(clk) THEN
        Enable_int <=   Enable6B & 
                        Enable6A & 
                        Enable5B & 
                        Enable5A & 
                        Enable4B & 
                        Enable4A & 
                        Enable3B & 
                        Enable3A & 
                        Enable2B & 
                        Enable2A & 
                        Enable1B & 
                        Enable1A;
                  
        CASE Enable_int IS 
           WHEN "000000000001" => Sel <= "0000"; en <= '1'; -- RS(00-01) / CH(1-4)
           WHEN "000000000010" => Sel <= "0001"; en <= '1'; -- RS(00-01) / CH(5-8)
           WHEN "000000000100" => Sel <= "0010"; en <= '1'; -- RS(02-03) / CH(1-4)
           WHEN "000000001000" => Sel <= "0011"; en <= '1'; -- RS(02-03) / CH(5-8)
           WHEN "000000010000" => Sel <= "0100"; en <= '1'; -- and so on..
           WHEN "000000100000" => Sel <= "0101"; en <= '1';
           WHEN "000001000000" => Sel <= "0110"; en <= '1';
           WHEN "000010000000" => Sel <= "0111"; en <= '1';
           WHEN "000100000000" => Sel <= "1000"; en <= '1';
           WHEN "001000000000" => Sel <= "1001"; en <= '1';
           WHEN "010000000000" => Sel <= "1010"; en <= '1';
           WHEN "100000000000" => Sel <= "1011"; en <= '1';
--           WHEN others         => Sel <= "1111"; en <= '0';
           WHEN others         => en <= '0';
        END CASE;
   END IF;
END PROCESS;
   
END RTL;
